package com.oracle.nio;

import sun.misc.SharedSecrets;
import sun.nio.ch.DirectBuffer;
import java.nio.Buffer;
import java.nio.ByteBuffer;
import sun.misc.JavaNioAccess;

public final class BufferSecrets<A>
{
    private static final JavaNioAccess javaNioAccess;
    private static final BufferSecrets<?> theBufferSecrets;
    
    private BufferSecrets() {
    }
    
    public static <A> BufferSecrets<A> instance() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new BufferSecretsPermission("access"));
        }
        return BufferSecrets.theBufferSecrets;
    }
    
    public ByteBuffer newDirectByteBuffer(long n, int i, A a) {
        if (i < 0) {
            throw new IllegalArgumentException("Negative capacity: " + i);
        }
        return BufferSecrets.javaNioAccess.newDirectByteBuffer(n, i, a);
    }
    
    public long address(Buffer buffer) {
        if ((buffer instanceof DirectBuffer)) {
            return ((DirectBuffer)buffer).address();
        }
        if (buffer == null) {
            throw new NullPointerException();
        }
        throw new UnsupportedOperationException();
    }
    
    public A attachment(Buffer buffer) {
        if ((buffer instanceof DirectBuffer)) {
            return ((DirectBuffer)buffer).attachment();
        }
        if (buffer == null) {
            throw new NullPointerException();
        }
        return null;
    }
    
    public void truncate(Buffer buffer) {
        BufferSecrets.javaNioAccess.truncate(buffer);
    }
    
    static {
        javaNioAccess = SharedSecrets.getJavaNioAccess();
        theBufferSecrets = new BufferSecrets();
    }
}
